/**
* @file main
*
*/

/*********************
*      INCLUDES
*********************/
#include <stdlib.h>
#include <Windows.h>
#include <SDL.h>
#include "lvgl/lvgl.h"
#include "lv_drivers/display/monitor.h"
#include "lv_drivers/indev/mouse.h"
#include "lv_drivers/indev/keyboard.h"
#include "lv_examples/lv_apps/demo/demo.h"
#include "lv_examples/lv_apps/benchmark/benchmark.h"
#include "lv_examples/lv_tests/lv_test_theme/lv_test_theme_1.h"
#include "lv_examples/lv_tutorial/10_keyboard/lv_tutorial_keyboard.h"

/*********************
*      DEFINES
*********************/

/**********************
*      TYPEDEFS
**********************/

/**********************
*  STATIC PROTOTYPES
**********************/
static void hal_init(void);
static int tick_thread(void *data);

/**********************
*  STATIC VARIABLES
**********************/
static lv_indev_t * kb_indev;

/**********************
*      MACROS
**********************/

#define LV_IMG_ZOOM_TEST_MODE (0)  // 0: drag mode  1: zoom mode

LV_IMG_DECLARE(test_ui);
LV_IMG_DECLARE(ui1);

/**********************
*   GLOBAL FUNCTIONS
**********************/

void img_zoom_task(lv_task_t* task)
{
    lv_obj_t *img = task->user_data;

    static int img_zoom_state = 0;
    static int img_zoom = 0;

    if( img_zoom == 0 ) img_zoom = lv_img_get_zoom(img);

    if (img_zoom_state == 0) {
        img_zoom += 10;
        if (img_zoom >= 390) {
            img_zoom_state = 1;
        }
    } else {
        img_zoom -= 10;
        if (img_zoom < 20) {
                img_zoom_state = 0;
        }
    }

    lv_img_set_zoom(img, img_zoom);
}

/* Color  RGB: 16bit Format565 */

int main(int argc, char** argv)
{
    /*Initialize LittlevGL*/
    lv_init();

    /*Initialize the HAL for LittlevGL*/
    hal_init();

    /*
     * Demo, benchmark, tests and tutorial.
     *
     * Uncomment any one (and only one) of the functions below to run that
     * particular demo, test or tutorial.
     */
    lv_obj_t *window = lv_page_create(lv_scr_act(), NULL);

    static lv_style_t window_style;
    lv_style_copy(&window_style, lv_obj_get_style(window));
    window_style.body.main_color = LV_COLOR_MAKE(150, 60, 80);
    window_style.body.grad_color = LV_COLOR_MAKE(150, 60, 80);
    lv_page_set_style(window, LV_PAGE_STYLE_BG, &window_style);
    lv_page_set_style(window, LV_PAGE_STYLE_SCRL, &window_style);

    lv_obj_set_size(window, LV_HOR_RES_MAX, LV_VER_RES_MAX);

#if LV_IMG_ZOOM_TEST_MODE
    #define LV_IMG_ORIGIN ui1
#else
    #define LV_IMG_ORIGIN test_ui
#endif

    lv_obj_t *IMG_Origin = lv_img_create(window, NULL);
    lv_img_set_src(IMG_Origin, &LV_IMG_ORIGIN);
    lv_obj_align(IMG_Origin, NULL, LV_ALIGN_CENTER, 0, 0);
    lv_obj_set_drag(IMG_Origin, true);

    lv_obj_t *IMG_Zoom = lv_img_create(window, IMG_Origin);
    lv_obj_align(IMG_Zoom, NULL, LV_ALIGN_CENTER, 0, 0);
    lv_img_set_antialias(IMG_Zoom, true);
    lv_img_set_zoom(IMG_Zoom, 390);

#if LV_IMG_ZOOM_TEST_MODE == 1
    lv_obj_set_hidden(IMG_Origin, true);
    lv_task_create(img_zoom_task, 100, LV_TASK_PRIO_MID, IMG_Zoom);
#else
    lv_obj_align(IMG_Zoom, IMG_Origin, LV_ALIGN_OUT_RIGHT_TOP, 10, 0);
#endif

    while (1) {
        /* Periodically call the lv_task handler.
        * It could be done in a timer interrupt or an OS task too.*/
        lv_task_handler();
        Sleep(10);       /*Just to let the system breathe */
    }

    return 0;
}

/**********************
*   STATIC FUNCTIONS
**********************/


/**
* Initialize the Hardware Abstraction Layer (HAL) for the Littlev graphics library
*/
static void hal_init(void)
{
    /* Add a display
    * Use the 'monitor' driver which creates window on PC's monitor to simulate a display*/
    monitor_init();

    lv_disp_drv_t disp_drv;
    lv_disp_drv_init(&disp_drv);            /*Basic initialization*/

    static lv_disp_buf_t disp_buf1;
    static lv_color_t buf1_1[LV_HOR_RES_MAX*10];
    lv_disp_buf_init(&disp_buf1, buf1_1, NULL, LV_HOR_RES_MAX*10);

    disp_drv.buffer = &disp_buf1;
    disp_drv.flush_cb = monitor_flush;
    lv_disp_drv_register(&disp_drv);

    /* Add the mouse (or touchpad) as input device
    * Use the 'mouse' driver which reads the PC's mouse*/
    mouse_init();
    lv_indev_drv_t indev_drv;
    lv_indev_drv_init(&indev_drv);          /*Basic initialization*/
    indev_drv.type = LV_INDEV_TYPE_POINTER;
    indev_drv.read_cb = mouse_read;         /*This function will be called periodically (by the library) to get the mouse position and state*/
    lv_indev_drv_register(&indev_drv);

    /* If the PC keyboard driver is enabled in`lv_drv_conf.h`
    * add this as an input device. It might be used in some examples. */
#if USE_KEYBOARD
    lv_indev_drv_t kb_drv;
    lv_indev_drv_init(&kb_drv);
    kb_drv.type = LV_INDEV_TYPE_KEYPAD;
    kb_drv.read_cb = keyboard_read;
    kb_indev = lv_indev_drv_register(&kb_drv);
#endif

    /* Tick init.
    * You have to call 'lv_tick_inc()' in every milliseconds
    * Create an SDL thread to do this*/
    SDL_CreateThread(tick_thread, "tick", NULL);
}

/**
* A task to measure the elapsed time for LittlevGL
* @param data unused
* @return never return
*/
static int tick_thread(void *data)
{
    while (1) {
        lv_tick_inc(5);
        SDL_Delay(5);   /*Sleep for 1 millisecond*/
    }

    return 0;
}
